@@ -22,13 +22,18 @@
  */
 
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <signal.h>
 #include <glade/glade.h>
 #include "zenity.h"
 #include "util.h"
 
 static guint timer;
 static GladeXML *glade_dialog;
-
+static ZenityData *zen_data;
 static GIOChannel *channel;
 
 gint zenity_progress_timeout (gpointer data);
@@ -97,7 +102,7 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
         /* We have a comment, so let's try to change the label */
         match = g_strstr_len (string->str, strlen (string->str), "#");
         match++;
-        gtk_label_set_text (GTK_LABEL (progress_label), g_strchomp (g_strchug (match)));
+        gtk_label_set_text (GTK_LABEL (progress_label), g_strcompress(g_strchomp (g_strchug (match))));
       } else {
 
         if (!g_ascii_isdigit (*(string->str)))
@@ -105,9 +110,18 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
 
         /* Now try to convert the thing to a number */
         percentage = atoi (string->str);
-        if (percentage > 100)
+        if (percentage >= 100) {
+	  GtkWidget *button;
+	  button = glade_xml_get_widget( glade_dialog,"zenity_progress_ok_button");
           gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 1.0);
-        else
+	  gtk_widget_set_sensitive(GTK_WIDGET (button), TRUE);
+	  gtk_widget_grab_focus(GTK_WIDGET (button));
+	  if (progress_data->autoclose) {
+		zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
+		gtk_main_quit();
+		
+	  }
+	} else
           gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), percentage / 100.0);
       }
 
@@ -159,12 +173,12 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
   GtkWidget *dialog;
   GtkWidget *text;
   GtkWidget *progress_bar;
-  guint input;
 
+  zen_data = data;
   glade_dialog = zenity_util_load_glade_file ("zenity_progress_dialog");
 
   if (glade_dialog == NULL) {
-    data->exit_code = -1;
+    data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
     return;
   }
 	
@@ -178,12 +192,10 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
   if (data->dialog_title)
     gtk_window_set_title (GTK_WINDOW (dialog), data->dialog_title);
 
-  if (data->window_icon)
-    zenity_util_set_window_icon (dialog, data->window_icon);
-  else
-    zenity_util_set_window_icon (dialog, ZENITY_IMAGE_FULLPATH ("zenity-progress.png"));
+  zenity_util_set_window_icon (dialog, data->window_icon, ZENITY_IMAGE_FULLPATH ("zenity-progress.png"));
 
-  gtk_window_set_default_size (GTK_WINDOW (dialog), data->width, data->height);
+  if (data->width > -1 || data->height > -1)
+    gtk_window_set_default_size (GTK_WINDOW (dialog), data->width, data->height);
 
   text = glade_xml_get_widget (glade_dialog, "zenity_progress_text");
   gtk_label_set_text (GTK_LABEL (text), progress_data->dialog_text);
@@ -193,8 +205,8 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
   if (progress_data->percentage > -1)
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 
                                    progress_data->percentage/100.0);
-	
-  gtk_widget_show (dialog);
+
+  zenity_util_show_dialog (dialog);
   zenity_progress_read_info (progress_data);
 
   gtk_main ();
@@ -203,24 +215,24 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
 static void
 zenity_progress_dialog_response (GtkWidget *widget, int response, gpointer data)
 {
-  ZenityData *zen_data = data;
-
   switch (response) {
     case GTK_RESPONSE_OK:
-      zen_data->exit_code = 0;
-      gtk_main_quit ();
+      zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
       break;
 		
     case GTK_RESPONSE_CANCEL:
-      /* FIXME: This should kill off the parent process - not entirely sure how to achieve this */
-      kill (0);
-      zen_data->exit_code = 1;
-      gtk_main_quit ();
+      /* FIXME: This should kill off the parent process nicely and return an error code
+       * I'm pretty sure there is a nice way to do this, but I'm clueless about this
+       * stuff. Should be using SIGHUP instead of 1 though.
+       */
+      kill (getppid (), 1);
+      zen_data->exit_code = zenity_util_return_exit_code (ZENITY_CANCEL);
       break;
   
     default:
       /* Esc dialog */
-      zen_data->exit_code = 1;
+      zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
       break;
   }
+  gtk_main_quit ();
 }
